#ifndef _MCDTRIANGLELIST_H
#define _MCDTRIANGLELIST_H
/* -*- mode: C; -*- */

/*
   Copyright (c) 1997-2002 MathEngine PLC

   $Name:  $

   Date: $Date: 2003/05/05 00:57:07 $ - Revision: $Revision: 1.1 $

   This software and its accompanying manuals have been developed
   by MathEngine PLC ("MathEngine") and the copyright and all other
   intellectual property rights in them belong to MathEngine. All
   rights conferred by law (including rights under international
   copyright conventions) are reserved to MathEngine. This software
   may also incorporate information which is confidential to
   MathEngine.

   Save to the extent permitted by law, or as otherwise expressly
   permitted by MathEngine, this software and the manuals must not
   be copied (in whole or in part), re-arranged, altered or adapted
   in any way without the prior written consent of the Company. In
   addition, the information contained in the software may not be
   disseminated without the prior written consent of MathEngine.

 */


#include <McdGeometry.h>
#include <McdGeometryTypes.h>

#ifdef __cplusplus
extern "C" {
#endif
    
    
MCD_DECLARE_GEOMETRY_TYPE( McdTriangleList );
    

/* Geometry given by user-specified triangle list */

MEPUBLIC
McdTriangleListID MEAPI McdTriangleListCreate(McdFramework *frame, 
                                              MeVector3 min,
                                              MeVector3 max,
                                              int triangleMaxCount,
                                              McdTriangleListFnPtr f);

MEPUBLIC
McdSphylID        MEAPI McdTriangleListInit(McdTriangleList *trilist, 
                                            McdFramework *frame, 
                                            MeVector3 min,
                                            MeVector3 max,
                                            int triangleMaxCount,
                                            McdTriangleListFnPtr f);

MEPUBLIC
void              MEAPI McdTriangleListTerm(McdTriangleListID trilist);

MEPUBLIC
void              MEAPI McdTriangleListDestroy(McdTriangleListID trilist);


MEPUBLIC
void              MEAPI McdTriangleListSetMaxTriangles(McdTriangleListID g, int max);

MEPUBLIC
int               MEAPI McdTriangleListGetMaxTriangles(McdTriangleListID g);

MEPUBLIC
void              MEAPI McdTriangleListSetBoundingBox(McdTriangleListID g,
                                                      MeVector3 min,
                                                      MeVector3 max);
MEPUBLIC
void              MEAPI McdTriangleListGetBoundingBox(McdTriangleListID g,
                                                      MeVector3 min,
                                                      MeVector3 max);
MEPUBLIC
void              MEAPI McdTriangleListSetGenerator(McdTriangleListID g,
                                                    McdTriangleListFnPtr f);
MEPUBLIC
McdTriangleListFnPtr MEAPI McdTriangleListGetGenerator(McdTriangleListID g);

MEPUBLIC
void              MEAPI McdTriangleListSetSweptFn(McdTriangleListID g,
                                                  McdTriangleListSweptFnPtr f);
MEPUBLIC
McdTriangleListSweptFnPtr MEAPI McdTriangleListGetSweptFn(McdTriangleListID g);

MEPUBLIC
void*             MEAPI McdTriangleListGetUserData( McdTriangleListID g );
MEPUBLIC
void              MEAPI McdTriangleListSetUserData( McdTriangleListID g, void* );

/*
    The following are found in McdTriangleListSafeTime.cpp
*/

MEPUBLIC
int MEAPI
McdGenericTriListSafeTime(McdModelPair* p, MeReal maxTime, 
                          McdSafeTimeResult *result);

/**    DEBUG ONLY!!

  This populates an array of n MeMatrix4 which are stepped forward
  in time from the present using timestep dt.
  On output, tm[0] is equal to mod->mInstance.mTM, tm[1] is one step
  forward, tm[2] is two steps, etc. up to n-1 steps.
*/
void McdGenerateTMSteps(MeMatrix4 *tm, const McdModelID mod, 
                        int n, MeReal dt);

/**   DEBUG ONLY!!

  This converts a UserTriangle to a convex geometry by sweeping it
  through space in proportion to the velocity and time step.
*/
void McdSweepTriangle(McdConvexHull *h,
                       const McdUserTriangle *tri, 
                       const MeVector3 vel, 
                       MeReal dt);

#ifdef __cplusplus
}
#endif

#endif /* _MCDTRIANGLELIST_H */
