#ifndef _MDTSKELETAL_H
#define _MDTSKELETAL_H
/* -*- mode: C; -*- */

/*
   Copyright (c) 1997-2002 MathEngine PLC

   $Name:  $

   Date: $Date: 2003/05/05 00:57:07 $ - Revision: $Revision: 1.1 $

   This software and its accompanying manuals have been developed
   by MathEngine PLC ("MathEngine") and the copyright and all other
   intellectual property rights in them belong to MathEngine. All
   rights conferred by law (including rights under international
   copyright conventions) are reserved to MathEngine. This software
   may also incorporate information which is confidential to
   MathEngine.

   Save to the extent permitted by law, or as otherwise expressly
   permitted by MathEngine, this software and the manuals must not
   be copied (in whole or in part), re-arranged, altered or adapted
   in any way without the prior written consent of the Company. In
   addition, the information contained in the software may not be
   disseminated without the prior written consent of MathEngine.

 */

/** @file
 * MdtSkeletal API functions.
 *
 */

#include <MePrecision.h>
#include <MdtTypes.h>

#ifdef __cplusplus
extern "C"
{
#endif


MEPUBLIC
MdtSkeletalID           MEAPI MdtSkeletalCreate(const MdtWorldID w);
MEPUBLIC
MdtSkeletalID           MEAPI MdtSkeletalInit(MdtSkeletal *const j,const MdtWorldID w);
MEPUBLIC
MdtConstraintID         MEAPI MdtSkeletalQuaConstraint(const MdtSkeletalID j);
MEPUBLIC
MdtSkeletalID           MEAPI MdtConstraintDCastSkeletal(const MdtConstraintID c);

#define                 MdtSkeletalDestroy(j) \
                            MdtConstraintDestroy(MdtSkeletalQuaConstraint(j))
#define                 MdtSkeletalTerm(j) \
                            MdtConstraintTerm(MdtSkeletalQuaConstraint(j))
#define                 MdtSkeletalEnable(j) \
                            MdtConstraintEnable(MdtSkeletalQuaConstraint(j))
#define                 MdtSkeletalDisable(j) \
                            MdtConstraintDisable(MdtSkeletalQuaConstraint(j))
#define                 MdtSkeletalIsEnabled(j) \
                            MdtConstraintIsEnabled(MdtSkeletalQuaConstraint(j))
#define                 MdtSkeletalSetSortKey(j,k) \
                            MdtConstraintSetSortKey(MdtSkeletalQuaConstraint(j),k)
#define                 MdtSkeletalGetSortKey(j) \
                            MdtConstraintGetSortKey(MdtSkeletalQuaConstraint(j))

/*
  Skeletal joint accessors.
*/

MEPUBLIC
MdtSkeletalCoords       MEAPI MdtSkeletalGetJoint(const MdtSkeletalID j);

MEPUBLIC
MdtSkeletalLimit        MEAPI MdtSkeletalGetHardLimits(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetHardLimitRadius(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetHardLimitTwistAngle(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetHardLimitSwingAngle1(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetHardLimitSwingAngle2(const MdtSkeletalID j);

MEPUBLIC
MdtSkeletalLimit        MEAPI MdtSkeletalGetSoftLimits(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetSoftLimitRadius(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetSoftLimitRadialStiffness(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetSoftLimitRadialDamping(const MdtSkeletalID j);

MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetSoftLimitTwistAngle(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetSoftLimitSwingAngle1(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetSoftLimitSwingAngle2(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetSoftLimitTwistStiffness(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetSoftLimitTwistDamping(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetSoftLimitSwingStiffness(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetSoftLimitSwingDamping(const MdtSkeletalID j);

MEPUBLIC
void                    MEAPI MdtSkeletalGetDrivePosition(const MdtSkeletalID j, MeVector3 pos);
MEPUBLIC
void                    MEAPI MdtSkeletalGetDriveLinearVel(const MdtSkeletalID j, MeVector3 vel);

MEPUBLIC
void                    MEAPI MdtSkeletalGetDriveQuaternion(const MdtSkeletalID j, MeVector4 quat);
MEPUBLIC
void                    MEAPI MdtSkeletalGetDriveAngularVel(const MdtSkeletalID j, MeVector3 w);

MEPUBLIC
MdtSkeletalDriveOptions MEAPI MdtSkeletalGetDriveLinear(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetDriveLinearStiffness(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetDriveLinearDamping(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetDriveLinearForceLimit(const MdtSkeletalID j);

MEPUBLIC
MdtSkeletalDriveOptions MEAPI MdtSkeletalGetDriveAngular(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetDriveTwistStiffness(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetDriveTwistDamping(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetDriveTwistForceLimit(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetDriveSwingStiffness(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetDriveSwingDamping(const MdtSkeletalID j);
MEPUBLIC
MeReal                  MEAPI MdtSkeletalGetDriveSwingForceLimit(const MdtSkeletalID j);


#define                 MdtSkeletalGetPosition(j, p) \
                            MdtConstraintGetPosition(MdtSkeletalQuaConstraint(j), p)
#define                 MdtSkeletalBodyGetAxis(j, i, a) \
                            MdtConstraintBodyGetAxis(MdtSkeletalQuaConstraint(j), i, a)
#define                 MdtSkeletalBodyGetAxes(j, i, p, o) \
                            MdtConstraintBodyGetAxes(MdtSkeletalQuaConstraint(j), i, p, o)

#define                 MdtSkeletalGetBody(j, bodyindex) \
                            MdtConstraintGetBody(MdtSkeletalQuaConstraint(j), bodyindex)
#define                 MdtSkeletalGetUserData(j) \
                            MdtConstraintGetUserData(MdtSkeletalQuaConstraint(j))
#define                 MdtSkeletalGetWorld(j) \
                            MdtConstraintGetWorld(MdtSkeletalQuaConstraint(j))
#define                 MdtSkeletalGetForce(j, bodyindex, f) \
                            MdtConstraintGetForce(MdtSkeletalQuaConstraint(j), bodyindex, f)
#define                 MdtSkeletalGetTorque(j, bodyindex, t) \
                            MdtConstraintGetTorque(MdtSkeletalQuaConstraint(j), bodyindex, t)

/*
  Skeletal joint mutators.
*/

MEPUBLIC
void                    MEAPI MdtSkeletalSetJoint(const MdtSkeletalID j,
                                         MdtSkeletalCoords hard);
MEPUBLIC
void                    MEAPI MdtSkeletalSetHardLimits(const MdtSkeletalID j, MdtSkeletalLimit hl);
MEPUBLIC
void                    MEAPI MdtSkeletalSetHardLimitSizes(const MdtSkeletalID j, MeReal radius,
                                         MeReal twist, MeReal swing1, MeReal swing2);
MEPUBLIC
void                    MEAPI MdtSkeletalSetHardLimitRadius(const MdtSkeletalID j, MeReal radius);
MEPUBLIC
void                    MEAPI MdtSkeletalSetHardLimitTwistAngle(const MdtSkeletalID j,
                                         MeReal twist);
MEPUBLIC
void                    MEAPI MdtSkeletalSetHardLimitSwingAngle1(const MdtSkeletalID j,
                                         MeReal swing1);
MEPUBLIC
void                    MEAPI MdtSkeletalSetHardLimitSwingAngle2(const MdtSkeletalID j,
                                         MeReal swing2);
MEPUBLIC
void                    MEAPI MdtSkeletalSetSoftLimits(const MdtSkeletalID j, MdtSkeletalLimit sl);
MEPUBLIC
void                    MEAPI MdtSkeletalSetSoftLimitSizes(const MdtSkeletalID j, MeReal radius,
                                         MeReal twist, MeReal swing1, MeReal swing2);
MEPUBLIC
void                    MEAPI MdtSkeletalSetSoftLimitRadius(const MdtSkeletalID j, MeReal radius);
MEPUBLIC
void                    MEAPI MdtSkeletalSetSoftLimitTwistAngle(const MdtSkeletalID j,
                                         MeReal twist);
MEPUBLIC
void                    MEAPI MdtSkeletalSetSoftLimitSwingAngle1(const MdtSkeletalID j,
                                         MeReal swing1);
MEPUBLIC
void                    MEAPI MdtSkeletalSetSoftLimitSwingAngle2(const MdtSkeletalID j,
                                         MeReal swing2);
MEPUBLIC
void                    MEAPI MdtSkeletalSetSoftLimitStiffness(const MdtSkeletalID j, MeReal lin,
                                         MeReal twist_stiff, MeReal swing_stiff);
MEPUBLIC
void                    MEAPI MdtSkeletalSetSoftLimitRadialStiffness(const MdtSkeletalID j,
                                         MeReal radius);
MEPUBLIC
void                    MEAPI MdtSkeletalSetSoftLimitTwistStiffness(const MdtSkeletalID j,
                                         MeReal twist);
MEPUBLIC
void                    MEAPI MdtSkeletalSetSoftLimitSwingStiffness(const MdtSkeletalID j,
                                         MeReal swing);
MEPUBLIC
void                    MEAPI MdtSkeletalSetSoftLimitDamping(const MdtSkeletalID j, MeReal lin,
                                         MeReal twist_damp, MeReal swing_damp);
MEPUBLIC
void                    MEAPI MdtSkeletalSetSoftLimitRadialDamping(const MdtSkeletalID j,
                                         MeReal radius);
MEPUBLIC
void                    MEAPI MdtSkeletalSetSoftLimitTwistDamping(const MdtSkeletalID j,
                                         MeReal twist);
MEPUBLIC
void                    MEAPI MdtSkeletalSetSoftLimitSwingDamping(const MdtSkeletalID j,
                                         MeReal swing);
MEPUBLIC
void                    MEAPI MdtSkeletalSetDrivePosition(const MdtSkeletalID j,const MeVector3 pos);
MEPUBLIC
void                    MEAPI MdtSkeletalSetDriveLinearVel(const MdtSkeletalID j,const MeVector3 vel);

MEPUBLIC
void                    MEAPI MdtSkeletalSetDriveQuaternion(const MdtSkeletalID j,const MeVector4 quat);
MEPUBLIC
void                    MEAPI MdtSkeletalSetDriveAngularVel(const MdtSkeletalID j,const MeVector3 w);

MEPUBLIC
void                    MEAPI MdtSkeletalSetDrives(const MdtSkeletalID j,
                                         MdtSkeletalDriveOptions lin, MdtSkeletalDriveOptions ang);
MEPUBLIC
void                    MEAPI MdtSkeletalSetDriveLinear(const MdtSkeletalID j,
                                         MdtSkeletalDriveOptions lin);
MEPUBLIC
void                    MEAPI MdtSkeletalSetDriveAngular(const MdtSkeletalID j,
                                         MdtSkeletalDriveOptions ang);
MEPUBLIC
void                    MEAPI MdtSkeletalSetDriveStiffness(const MdtSkeletalID j,
                                         MeReal linear, MeReal twist, MeReal swing);
MEPUBLIC
void                    MEAPI MdtSkeletalSetDriveLinearStiffness(const MdtSkeletalID j,
                                         MeReal linear);
MEPUBLIC
void                    MEAPI MdtSkeletalSetDriveTwistStiffness(const MdtSkeletalID j,
                                         MeReal twist);
MEPUBLIC
void                    MEAPI MdtSkeletalSetDriveSwingStiffness(const MdtSkeletalID j,
                                         MeReal swing);
MEPUBLIC
void                    MEAPI MdtSkeletalSetDriveDamping(const MdtSkeletalID j,
                                         MeReal linear, MeReal twist, MeReal swing);
MEPUBLIC
void                    MEAPI MdtSkeletalSetDriveLinearDamping(const MdtSkeletalID j,
                                         MeReal linear);
MEPUBLIC
void                    MEAPI MdtSkeletalSetDriveTwistDamping(const MdtSkeletalID j,
                                         MeReal twist);
MEPUBLIC
void                    MEAPI MdtSkeletalSetDriveSwingDamping(const MdtSkeletalID j,
                                         MeReal swing);
MEPUBLIC
void                    MEAPI MdtSkeletalSetDriveForceLimit(const MdtSkeletalID j,
                                         MeReal linear, MeReal twist, MeReal swing);
MEPUBLIC
void                    MEAPI MdtSkeletalSetDriveLinearForceLimit(const MdtSkeletalID j,
                                         MeReal linear);
MEPUBLIC
void                    MEAPI MdtSkeletalSetDriveTwistForceLimit(const MdtSkeletalID j,
                                         MeReal twist);
MEPUBLIC
void                    MEAPI MdtSkeletalSetDriveSwingForceLimit(const MdtSkeletalID j,
                                         MeReal swing);

MEPUBLIC
void                    MEAPI MdtSkeletalZeroTwist(const MdtSkeletalID j);

#define                 MdtSkeletalSetPosition(j, x, y, z) \
                            MdtConstraintSetPosition(MdtSkeletalQuaConstraint(j), x, y, z)
#define                 MdtSkeletalSetAxes(j, px, py, pz, ox, oy, oz) \
                            MdtConstraintSetAxes(MdtSkeletalQuaConstraint(j), px, py, pz, ox, oy, oz)
#define                 MdtSkeletalBodySetAxes(j, i, px, py, pz, ox, oy, oz) \
                            MdtConstraintBodySetAxes(MdtSkeletalQuaConstraint(j), i, px, py, pz, ox, oy, oz)

#define                 MdtSkeletalSetBodies(j, b1, b2) \
                            MdtConstraintSetBodies(MdtSkeletalQuaConstraint(j), b1, b2)
#define                 MdtSkeletalSetUserData(j, d) \
                            MdtConstraintSetUserData(MdtSkeletalQuaConstraint(j), d)


#ifdef __cplusplus
}
#endif


#endif
