#ifndef _MEASSETDBTYPES__H /* -*- mode: C; -*- */
#define _MEASSETDBTYPES__H

/*
   Copyright (c) 1997-2002 MathEngine PLC

   $Name:  $

   Date: $Date: 2003/05/05 00:57:07 $ - Revision: $Revision: 1.1 $

   This software and its accompanying manuals have been developed
   by MathEngine PLC ("MathEngine") and the copyright and all other
   intellectual property rights in them belong to MathEngine. All
   rights conferred by law (including rights under international
   copyright conventions) are reserved to MathEngine. This software
   may also incorporate information which is confidential to
   MathEngine.

   Save to the extent permitted by law, or as otherwise expressly
   permitted by MathEngine, this software and the manuals must not
   be copied (in whole or in part), re-arranged, altered or adapted
   in any way without the prior written consent of the Company. In
   addition, the information contained in the software may not be
   disseminated without the prior written consent of MathEngine.

 */

/** @file
 * Types associated with MeAssetDB.
 */

#include <MePrecision.h>
#include <MeMisc.h>
#include <MeIDPool.h>
#include <MeHash.h>

/* Max properties */
#define MAX_JOINT_FLOAT_PROPERTIES          23
#define MAX_JOINT_INT_PROPERTIES            5
#define MAX_JOINT_BOOL_PROPERTIES           2

/* ball and socket properties */
#define N_BSJOINT_FLOAT_PROPERTIES          0
#define N_BSJOINT_INT_PROPERTIES            0
#define N_BSJOINT_BOOL_PROPERTIES           0

/* fixedpath properties */
#define N_FPJOINT_FLOAT_PROPERTIES          0
#define N_FPJOINT_INT_PROPERTIES            0
#define N_FPJOINT_BOOL_PROPERTIES           0

/* universal joint properties */
#define N_UNIVERSAL_FLOAT_PROPERTIES        0
#define N_UNIVERSAL_INT_PROPERTIES          0
#define N_UNIVERSAL_BOOL_PROPERTIES         0

/* hinge properties */
#define N_HINGE_FLOAT_PROPERTIES            6
#define N_HINGE_INT_PROPERTIES              0
#define N_HINGE_BOOL_PROPERTIES             2

#define HINGE_LIMIT_UPPER_STOP_1F           0 
#define HINGE_LIMIT_LOWER_STOP_1F           1 
#define HINGE_LIMIT_UPPER_STIFF_1F          2 
#define HINGE_LIMIT_LOWER_STIFF_1F          3 
#define HINGE_LIMIT_DESIRED_VEL_1F          4 
#define HINGE_LIMIT_FORCE_LIMIT_1F          5 
#define HINGE_LIMIT_ACTIVE_1B               0 
#define HINGE_LIMIT_MOTORIZED_1B            1 

/* carwheel properties */
#define N_CARWHEEL_FLOAT_PROPERTIES         10
#define N_CARWHEEL_INT_PROPERTIES           0
#define N_CARWHEEL_BOOL_PROPERTIES          1

#define CARWHEEL_HINGE_MOTOR_DESIRED_VEL_1F 0 
#define CARWHEEL_HINGE_MOTOR_FORCE_LIMIT_1F 1 
#define CARWHEEL_STEER_MOTOR_DESIRED_VEL_1F 2 
#define CARWHEEL_STEER_MOTOR_FORCE_LIMIT_1F 3 
#define CARWHEEL_SUSP_STIFF_1F              4 
#define CARWHEEL_SUSP_DAMP_1F               5 
#define CARWHEEL_SUSP_UPPER_LIMIT_1F        6 
#define CARWHEEL_SUSP_LOWER_LIMIT_1F        7 
#define CARWHEEL_SUSP_REF_1F                8 
#define CARWHEEL_SUSP_SOFT_1F               9
#define CARWHEEL_STEER_LOCK_1B              0 

/* cone limit properties */
#define N_CONELIMIT_FLOAT_PROPERTIES        2
#define N_CONELIMIT_INT_PROPERTIES          0
#define N_CONELIMIT_BOOL_PROPERTIES         0

#define CONELIMIT_HALF_ANGLE_1F             0 
#define CONELIMIT_STIFF_1F                  1 
 
/* rpro properties */
#define N_RPRO_FLOAT_PROPERTIES             6
#define N_RPRO_INT_PROPERTIES               0
#define N_RPRO_BOOL_PROPERTIES              0

#define RPRO_LINEAR_STRENGTH1_1F            0
#define RPRO_LINEAR_STRENGTH2_1F            1
#define RPRO_LINEAR_STRENGTH3_1F            2
#define RPRO_ANGULAR_STRENGTH1_1F           3
#define RPRO_ANGULAR_STRENGTH2_1F           4
#define RPRO_ANGULAR_STRENGTH3_1F           5
/* and for convenience... */
#define RPRO_LINEAR_STRENGTH_3F             0 
#define RPRO_ANGULAR_STRENGTH_3F            3 

/* prismatic properties */
#define N_PRISMATIC_FLOAT_PROPERTIES        6
#define N_PRISMATIC_INT_PROPERTIES          0
#define N_PRISMATIC_BOOL_PROPERTIES         2

#define PRISMATIC_LIMIT_UPPER_STOP_1F       0 
#define PRISMATIC_LIMIT_LOWER_STOP_1F       1 
#define PRISMATIC_LIMIT_UPPER_STIFF_1F      2 
#define PRISMATIC_LIMIT_LOWER_STIFF_1F      3 
#define PRISMATIC_LIMIT_DESIRED_VEL_1F      4
#define PRISMATIC_LIMIT_FORCE_LIMIT_1F      5 
#define PRISMATIC_LIMIT_ACTIVE_1B           0 
#define PRISMATIC_LIMIT_MOTORIZED_1B        1

/* skeletal properties */
#define N_SKELETAL_FLOAT_PROPERTIES                    23
#define N_SKELETAL_INT_PROPERTIES                      5
#define N_SKELETAL_BOOL_PROPERTIES                     0

#define SKELETAL_JOINT_HARD_TYPE_1I                    0
#define SKELETAL_HARD_LIMIT_TYPE_1I                    1
#define SKELETAL_SOFT_LIMIT_TYPE_1I                    2
#define SKELETAL_ANGULAR_DRIVE_TYPE_1I                 3
#define SKELETAL_LINEAR_DRIVE_TYPE_1I                  4

#define SKELETAL_HARD_LIMIT_RADIUS_1F                  0
#define SKELETAL_HARD_LIMIT_TWIST_ANGLE_1F             1
#define SKELETAL_HARD_LIMIT_SWING_ANGLE1_1F            2
#define SKELETAL_HARD_LIMIT_SWING_ANGLE2_1F            3
#define SKELETAL_SOFT_LIMIT_RADIUS_1F                  4
#define SKELETAL_SOFT_LIMIT_RADIAL_STIFF_1F            5
#define SKELETAL_SOFT_LIMIT_RADIAL_DAMP_1F             6
#define SKELETAL_SOFT_LIMIT_TWIST_ANGLE_1F             7
#define SKELETAL_SOFT_LIMIT_SWING_ANGLE1_1F            8
#define SKELETAL_SOFT_LIMIT_SWING_ANGLE2_1F            9
#define SKELETAL_SOFT_LIMIT_TWIST_STIFF_1F             10
#define SKELETAL_SOFT_LIMIT_TWIST_DAMP_1F              11
#define SKELETAL_SOFT_LIMIT_SWING_STIFF_1F             12
#define SKELETAL_SOFT_LIMIT_SWING_DAMP_1F              13
#define SKELETAL_DRIVE_LINEAR_STIFF_1F                 14
#define SKELETAL_DRIVE_LINEAR_DAMP_1F                  15
#define SKELETAL_DRIVE_LINEAR_FORCE_LIMIT_1F           16
#define SKELETAL_DRIVE_TWIST_STIFF_1F                  17
#define SKELETAL_DRIVE_TWIST_DAMP_1F                   18
#define SKELETAL_DRIVE_TWIST_FORCE_LIMIT_1F            19
#define SKELETAL_DRIVE_SWING_STIFF_1F                  20
#define SKELETAL_DRIVE_SWING_DAMP_1F                   21
#define SKELETAL_DRIVE_SWING_FORCE_LIMIT_1F            22

/* skeletal options */

#define SKELETAL_OPTION_TWIST                           1
#define SKELETAL_OPTION_SWING1                          2
#define SKELETAL_OPTION_SWING2                          4
#define SKELETAL_OPTION_POS0                            256
#define SKELETAL_OPTION_POS1                            512
#define SKELETAL_OPTION_POS2                            1024
#define SKELETAL_OPTION_SWING                           6
#define SKELETAL_OPTION_ANGULAR                         7
#define SKELETAL_OPTION_LINEAR                          1792

#define SKELETAL_LIMIT_NONE                             0
#define SKELETAL_LIMIT_TWIST                            1
#define SKELETAL_LIMIT_SWING                            6
#define SKELETAL_LIMIT_TWIST_AND_SWING                  7
#define SKELETAL_LIMIT_SPHERICAL                        1792

#define SKELETAL_DRIVE_POSITION                         1
#define SKELETAL_DRIVE_VELOCITY                         2
#define SKELETAL_DRIVE_FORCE_LIMIT                      4
#define SKELETAL_DRIVE_ISO                              8

/* angular3 properties */
#define N_ANGULAR3_FLOAT_PROPERTIES         2
#define N_ANGULAR3_INT_PROPERTIES           0
#define N_ANGULAR3_BOOL_PROPERTIES          1

#define ANGULAR3_STIFFNESS_1F               0
#define ANGULAR3_DAMPING_1F                 1
#define ANGULAR3_ROT_ENABLED_1B             0

/* spring6 properties */
#define N_SPRING6_FLOAT_PROPERTIES          12
#define N_SPRING6_INT_PROPERTIES            0
#define N_SPRING6_BOOL_PROPERTIES           0

#define SPRING6_LINEAR_STIFF1_1F            0
#define SPRING6_LINEAR_STIFF2_1F            1
#define SPRING6_LINEAR_STIFF3_1F            2
#define SPRING5_ANGULAR_STIFF1_1F           3
#define SPRING5_ANGULAR_STIFF2_1F           4
#define SPRING5_ANGULAR_STIFF3_1F           5
#define SPRING6_LINEAR_DAMP1_1F             6
#define SPRING6_LINEAR_DAMP2_1F             7
#define SPRING6_LINEAR_DAMP3_1F             8
#define SPRING5_ANGULAR_DAMP1_1F            9
#define SPRING5_ANGULAR_DAMP2_1F            10
#define SPRING5_ANGULAR_DAMP3_1F            11
/* and for convenience... */
#define SPRING6_LINEAR_STIFF_3F             0
#define SPRING5_ANGULAR_STIFF_3F            3
#define SPRING6_LINEAR_DAMP_3F              6
#define SPRING5_ANGULAR_DAMP_3F             9

/*@{*/
/** */
typedef struct MyMesh               MyMesh;
typedef struct MyAABB               MyAABB;

typedef struct MeAssetDB            MeAssetDB;
typedef struct MeFPrimitive         MeFPrimitive;
typedef struct MeFGeometry          MeFGeometry;
typedef struct MeFJoint             MeFJoint;
typedef struct MeFJointParams       MeFJointParams;
typedef struct MeFAsset             MeFAsset;
typedef struct MeFAssetPart         MeFAssetPart;
/*@}*/

/*@{*/
/** iterators */
typedef struct MeFVertexIt          MeFVertexIt;
typedef struct MeFPrimitiveIt       MeFPrimitiveIt;
typedef struct MeFGeometryIt        MeFGeometryIt;
typedef struct MeFJointIt           MeFJointIt;
typedef struct MeFAssetPartIt       MeFAssetPartIt;
typedef struct MeFAssetIt           MeFAssetIt;
/*@}*/

/*@{*/
/** nodes */
typedef struct MeFPrimitiveNode     MeFPrimitiveNode;
typedef struct MeFGeometryNode      MeFGeometryNode;
typedef struct MeFJointNode         MeFJointNode;
typedef struct MeFAssetPartNode     MeFAssetPartNode;
typedef struct MeFAssetNode         MeFAssetNode;

typedef enum
{
    kMeFPrimitiveTypeUnknown = 0,
    kMeFPrimitiveTypeSphere,
    kMeFPrimitiveTypeBox,
    kMeFPrimitiveTypeCylinder,
    kMeFPrimitiveTypeSphyl,
    kMeFPrimitiveTypePlane,
    kMeFPrimitiveTypeConvex
} MeFPrimitiveType;

typedef enum
{
    kMeFJointTypeUnknown = 0,
    kMeFJointTypeCarwheel,
    kMeFJointTypeHinge,
    kMeFJointTypeBallAndSocket,
    kMeFJointTypeConeLimit,
    kMeFJointTypeUniversal,
    kMeFJointTypeRpro,
    kMeFJointTypePrismatic,
    kMeFJointTypeSkeletal,
    kMeFJointTypeAngular3,
    kMeFJointTypeSpring6,
    kMeFJointTypeFixedPath
} MeFJointType;

typedef enum
{
    kMeFAssetPartTypeDynamicsAndGeometry,
    kMeFAssetPartTypeDynamicsOnly,
    kMeFAssetPartTypeGeometryOnly
} MeFAssetPartType;


struct MeFVertexIt
{
    int               currentVertex;
    MeFPrimitive      *prim;
};

struct MeFPrimitive
{
    char              *id;
    MeFGeometry       *geometry; /* pointer to parent */
    MeVector3         *vertices;
    int               nVertices;
    int               maxVertices;
    MeFPrimitiveType  type;
    MeVector3         dims; /* dims[0] is radius, dims[1] is height */
    MeMatrix4         tm;
};

struct MeFPrimitiveNode
{
    MeFPrimitiveNode  *prev;
    MeFPrimitive      *current;
    MeFPrimitiveNode  *next;
};

struct MeFPrimitiveIt
{
    MeFPrimitiveNode *node;
};

struct MeFGeometry
{
    char              *id;
    MeFAsset          *asset; /* pointer to parent */
    MeFPrimitiveNode  *nilPrimitive;
    char              *graphicHint;
    MeReal            graphicScale;
    MeVector3         graphicOffset;
    int               primCount;
};

struct MeFGeometryNode
{
    MeFGeometryNode   *prev;
    MeFGeometry       *current;
    MeFGeometryNode   *next;
};

struct MeFGeometryIt
{
    MeFGeometryNode *node;
};

/* joint-specific data */
struct MeFJointParams
{
    MeReal          _float[MAX_JOINT_FLOAT_PROPERTIES];   
    MeBool          _bool[MAX_JOINT_BOOL_PROPERTIES];
    int             _int[MAX_JOINT_INT_PROPERTIES];  
};

/* generic joint data */
struct MeFJoint
{
    char            *id;
    char            *part[2];
    MeFAsset        *asset; /* pointer to parent */
    MeFJointType    type;
    MeVector3       pos[2];
    MeVector3       pax[2];
    MeVector3       oax[2];
    MeFJointParams  params;
};

struct MeFJointNode
{
    MeFJointNode  *prev;
    MeFJoint      *current;
    MeFJointNode  *next;
};

struct MeFJointIt
{
    MeFJointNode  *node;
};

struct MeFAssetPart
{
    char          *id;
    MeFAssetPartType  type;
    MeFAsset      *asset; /* pointer to parent */
    char          *geometry;
    MeVector3     mass_offset;
    MeReal        mass;
    MeReal        density;
    MeReal        inertia[6];
    MeReal        linearDamp;
    MeReal        angularDamp;
    MeVector3     fastSpin;
    MeBool        useFastSpin;
    char          *graphicHint;
    char          *parent;
    MeMatrix4     tm;
    MeReal        graphicScale;
    MeVector3     graphicOffset;
    int           index; /* index into disabled collisions array */
};

struct MeFAssetPartNode
{
    MeFAssetPartNode  *prev;
    MeFAssetPart      *current;
    MeFAssetPartNode  *next;
};

struct MeFAssetPartIt
{
    MeFAssetPartNode  *node;
};

/**
 * Asset data structure.
 */
struct MeFAsset
{
    char              *name;                /** Name of the asset. */
    MeAssetDB         *db;                  /* pointer to parent, in this case the asset database */
    MeFGeometryNode   *nilGeometry;         /** Geometries */
    MeFAssetPartNode  *nilPart;             /** Parts */
    MeFJointNode      *nilJoint;            /** Joints */
    MeHash            *nameToGeometry;      
    MeHash            *nameToPart;
    MeHash            *nameToJoint;
    char              *graphicHint;
    char              *refPart;
    MeU32             *disabledColArray;
    MeIDPool          *disabledColIndexPool;   /* used in disabled collisions array */
    int               id;
    MeReal            graphicScale;
    int               geomCount;
    int               jointCount;
    int               partCount;
    int               maxParts;
    MeReal            massScale;
    MeReal            lengthScale;
};

struct MeFAssetNode
{
    MeFAssetNode  *prev;
    MeFAsset      *current;
    MeFAssetNode  *next;
};

struct MeFAssetIt
{
    MeFAssetNode  *node;
};

struct MeAssetDB
{
    MeFAssetNode    *nilAsset;
    unsigned int    assetCount;
};

struct MyMesh
{
    int         (*faceVertex)[3];
    MeVector3   *verts;
    int         numFaces;
    int         numVerts;
};

struct MyAABB
{
    MeVector3 min;
    MeVector3 max;
    MeVector3 size;
    MeVector3 centre;
};

/*@}*/
#endif
